﻿module Snake

open System

//====================================================
// Подробное описание работы классов и методов исходника на:
// https://www.interestprograms.ru
// Исходные коды программ и игр
//====================================================


// Элемент-звено змейки, содержит свои координаты
type pos = { row: int; col: int }



type Snake() =
    let symbolChain = "●"
    let symbolHead = "○"

    let mutable listPos: pos list = []

    let lastIndex () = listPos.Length - 1

    let drawHead () =
        Console.SetCursorPosition(listPos.[lastIndex ()].row, listPos.[lastIndex ()].col)
        Console.Write(symbolHead)

    let drawSnake () =
        for d in listPos do
            Console.SetCursorPosition(d.row, d.col)
            Console.Write(symbolChain)

        drawHead ()

    let erase () =
        Console.SetCursorPosition(listPos.Item(0).row, listPos.Item(0).col)
        Console.Write(" ")

    // Передвижение на один шаг змейки в требуемом направлении.
    let moveSnake direction =
        let posHead = listPos.Item(lastIndex ())

        let mutable x = 0
        let mutable y = 0

        match direction with
        | 1 when listPos.[lastIndex ()].row < (Console.WindowWidth - 1) -> x <- 1
        | 2 when listPos.[lastIndex ()].row > 0 -> x <- -1
        | 3 when listPos.[lastIndex ()].col < (Console.WindowHeight - 1) -> y <- 1
        | 4 when listPos.[lastIndex ()].col > 0 -> y <- -1
        | _ -> ()

        // Двигаемся только если получено направление.
        if x <> 0 || y <> 0 then
            erase ()

            let posNew =
                { row = posHead.row + x
                  col = posHead.col + y }

            // Удаляем хвостовое звено, голову передвигаем на новое место
            // и тем самым перемещаем все звенья на один шаг вперёд.
            listPos <- listPos.Tail @ [ posNew ]

            // Рисуем змейку
            drawSnake ()

    // Инициализация цепочки символов и рисование змейки.
    do
        for i = 0 to 30 do
            let pos = { row = i; col = 0 }
            listPos <- listPos @ [ pos ]

        drawSnake ()

    // Открытый метод для движения змейки.
    member x.Move(dir: int) = moveSnake dir

    // Открытое свойство получения текущей координаты головы змейки.
    member x.Pos = listPos.[lastIndex ()]
